package gui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.opencv.core.Mat;
import org.opencv.core.Rect;

import regions.Line;
import regions.Resizable;
import regions.ResizableManager;
import regions.type.RegionType;
import segmentation.ImageProcessor;

@SuppressWarnings("serial")
public class BackgroundPanel extends JPanel {

	private static final int THICKNESS = 1;

	private Mat imageMat;
	private Image image;
	// private ResizableManager resizableManager;
	private BackgroundPanel pnl;
	
	private ResizableManager resizableManager;

	private ArrayList<Resizable> positions;
	private ArrayList<Line> lines;

	private Resizable processingResizable;
	private Resizable activeResizable;
	private Rectangle activeRectangle;

	private boolean drawingRectangle = false;
	private boolean drawingLine = false;

	private Point lineStartPoint;
	private Point lineEndPoint;

	private Line line;

	// ShapeMover
	int preX;
	int preY;
	boolean isFirstTime = true;
	boolean pressOut = false;
	Rectangle area;

	// Resizer
	boolean dragging = false;
	final int PROX_DIST = 3;

	public BackgroundPanel(Mat image, ResizableManager resizableManager) {
		updateImage(image);
		setOpaque(false);
		setPnl(this);

		setResizableManager(resizableManager);
		setPositions(resizableManager.getPositions());
		setLines(resizableManager.getLines());

		addListeners();
	}

	public void addListeners() {
		addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					drawingRectangle = false;
					drawingLine = true;

					lineStartPoint = new Point(e.getX(), e.getY());
				}
			}

			@Override
			public void mousePressed(MouseEvent e) {
				// System.out.println("pressed");
				if (SwingUtilities.isLeftMouseButton(e)) {
					activeResizable = ResizableManager.identifyActiveRect(e, positions);
					if (activeResizable != null) {
						activeRectangle = activeResizable.getRect().getBounds();
					} else {
						ResizableManager.manageActive(null, positions);
					}
					repaint();

					if (pnl.getCursor() != Cursor.getDefaultCursor()) {
						// If cursor is set for resizing, allow dragging.
						dragging = true;
					} else {
						if (activeRectangle != null && activeRectangle.contains(e.getX(), e.getY())) {
							preX = (int) activeRectangle.getX() - e.getX();
							preY = (int) activeRectangle.getY() - e.getY();

							updateLocation(e);
						} else {
							pressOut = true;
						}
					}
				} else {
					activeResizable = ResizableManager.identifyActiveRect(e, positions);

					if (activeResizable != null) {
						positions.remove(activeResizable);
						activeResizable = null;
					} else {
						setActiveRectangle(new Rectangle(e.getX(), e.getY(), 100, 50));
						setActiveResizable(new Resizable(activeRectangle, RegionType.PARAGRAPH));
					}

					drawingRectangle = true;
				}
			}

			public void mouseReleased(MouseEvent e) {
				// System.out.println("released");
				if (drawingRectangle && activeResizable != null) {
					double brX = activeResizable.getRect().getX() + activeResizable.getRect().getWidth();
					double brY = activeResizable.getRect().getY() + activeResizable.getRect().getHeight();

					if (brX > image.getWidth(null) - 1) {
						brX = image.getWidth(null) - 1;
					}
					if (brY > image.getHeight(null) - 1) {
						brY = image.getHeight(null) - 1;
					}

					double width = brX - activeResizable.getRect().getX();
					double height = brY - activeResizable.getRect().getY();

					Resizable newRes = new Resizable(new Rectangle.Double(activeResizable.getRect().getX(),
							activeResizable.getRect().getY(), width, height), activeResizable.getType());
					newRes.setVisible(true);
					newRes.setActive(true);
					ResizableManager.manageActive(newRes, positions);
					positions.add(newRes);
					setActiveResizable(newRes);
				}

				if (drawingLine) {
					line = new Line(lineStartPoint, lineEndPoint, imageMat);
					lines.add(line);

					lineStartPoint = null;
					lineEndPoint = null;
				}

				drawingRectangle = false;
				drawingLine = false;
				pressOut = false;
				dragging = false;

				repaint();
			}
		});

		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(MouseEvent e) {
				// System.out.println("dragged");
				if (dragging) {
					Point p = e.getPoint();

					if (area.contains(p)) {
						Rectangle r = activeRectangle.getBounds();

						int type = getCursor().getType();
						int dx = p.x - r.x;
						int dy = p.y - r.y;

						switch (type) {
						case Cursor.N_RESIZE_CURSOR:
							int height = r.height - dy;
							activeRectangle.setRect(r.x, r.y + dy, r.width, height);
							break;
						case Cursor.NW_RESIZE_CURSOR:
							int width = r.width - dx;
							height = r.height - dy;
							activeRectangle.setRect(r.x + dx, r.y + dy, width, height);
							break;
						case Cursor.W_RESIZE_CURSOR:
							width = r.width - dx;
							activeRectangle.setRect(r.x + dx, r.y, width, r.height);
							break;
						case Cursor.SW_RESIZE_CURSOR:
							width = r.width - dx;
							height = dy;
							activeRectangle.setRect(r.x + dx, r.y, width, height);
							break;
						case Cursor.S_RESIZE_CURSOR:
							height = dy;
							activeRectangle.setRect(r.x, r.y, r.width, height);
							break;
						case Cursor.SE_RESIZE_CURSOR:
							width = dx;
							height = dy;
							activeRectangle.setRect(r.x, r.y, width, height);
							break;
						case Cursor.E_RESIZE_CURSOR:
							width = dx;
							activeRectangle.setRect(r.x, r.y, width, r.height);
							break;
						case Cursor.NE_RESIZE_CURSOR:
							width = dx;
							height = r.height - dy;
							activeRectangle.setRect(r.x, r.y + dy, width, height);
							break;
						default:
							System.out.println("unexpected type: " + type);
						}

						repaint();
					}
				} else if (drawingRectangle) {
					if(activeRectangle != null) {
					setActiveResizable(new Resizable(new Rect((int) activeRectangle.getX(),
							(int) activeRectangle.getY(), (int) (e.getX() - activeRectangle.getX()),
							(int) (e.getY() - activeRectangle.getY())), RegionType.PARAGRAPH));
					}
					repaint();
				} else if (!pressOut) {
					updateLocation(e);
				}
			}

			public void mouseMoved(MouseEvent e) {
				if (drawingLine) {
					lineEndPoint = new Point(e.getX(), e.getY());
					repaint();
				} else {
					if (activeRectangle != null) {
						Point p = e.getPoint();
						if (!isOverRect(p)) {
							if (getCursor() != Cursor.getDefaultCursor()) {
								// If cursor is not over rect reset it to the
								// default.
								setCursor(Cursor.getDefaultCursor());
							}
							return;
						}
						// Locate cursor relative to center of rect.
						int outcode = getOutcode(p);
						Rectangle r = activeRectangle.getBounds();
						switch (outcode) {
						case Rectangle.OUT_TOP:
							if (Math.abs(p.y - r.y) < PROX_DIST) {
								setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
							}
							break;
						case Rectangle.OUT_TOP + Rectangle.OUT_LEFT:
							if (Math.abs(p.y - r.y) < PROX_DIST && Math.abs(p.x - r.x) < PROX_DIST) {
								setCursor(Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR));
							}
							break;
						case Rectangle.OUT_LEFT:
							if (Math.abs(p.x - r.x) < PROX_DIST) {
								setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
							}
							break;
						case Rectangle.OUT_LEFT + Rectangle.OUT_BOTTOM:
							if (Math.abs(p.x - r.x) < PROX_DIST && Math.abs(p.y - (r.y + r.height)) < PROX_DIST) {
								setCursor(Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR));
							}
							break;
						case Rectangle.OUT_BOTTOM:
							if (Math.abs(p.y - (r.y + r.height)) < PROX_DIST) {
								setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
							}
							break;
						case Rectangle.OUT_BOTTOM + Rectangle.OUT_RIGHT:
							if (Math.abs(p.x - (r.x + r.width)) < PROX_DIST
									&& Math.abs(p.y - (r.y + r.height)) < PROX_DIST) {
								setCursor(Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
							}
							break;
						case Rectangle.OUT_RIGHT:
							if (Math.abs(p.x - (r.x + r.width)) < PROX_DIST) {
								setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
							}
							break;
						case Rectangle.OUT_RIGHT + Rectangle.OUT_TOP:
							if (Math.abs(p.x - (r.x + r.width)) < PROX_DIST && Math.abs(p.y - r.y) < PROX_DIST) {
								setCursor(Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR));
							}
							break;
						default: // center
							setCursor(Cursor.getDefaultCursor());
						}
					}
				}
			}
		});
	}

	public void updateLocation(MouseEvent e) {
		if (activeRectangle != null) {
			activeRectangle.setRect(preX + e.getX(), preY + e.getY(), activeRectangle.getWidth(),
					activeRectangle.getHeight());
			checkRect();
			repaint();
		}
	}

	private int getOutcode(Point p) {
		Rectangle r = (Rectangle) activeRectangle.getBounds().clone();
		r.grow(-PROX_DIST, -PROX_DIST);

		return r.outcode(p.x, p.y);
	}

	private boolean isOverRect(Point p) {
		Rectangle r = (Rectangle) activeRectangle.getBounds().clone();
		r.grow(PROX_DIST, PROX_DIST);

		return r.contains(p);
	}

	private boolean checkRect() {
		if (area == null) {
			return false;
		}

		if (area.contains(activeRectangle.getX(), activeRectangle.getY(), activeRectangle.getWidth(),
				activeRectangle.getHeight())) {
			return true;
		}

		int new_x = (int) activeRectangle.getX();
		int new_y = (int) activeRectangle.getY();

		if ((activeRectangle.getX() + activeRectangle.getWidth()) > area.getWidth()) {
			new_x = (int) area.getWidth() - (int) (activeRectangle.getWidth() - 1);
		}
		if (activeRectangle.getX() < 0) {
			new_x = -1;
		}
		if ((activeRectangle.getY() + activeRectangle.getHeight()) > area.getHeight()) {
			new_y = (int) area.getHeight() - (int) (activeRectangle.getHeight() - 1);
		}
		if (activeRectangle.getY() < 0) {
			new_y = -1;
		}

		// activeRectangle.setLocation(new_x, new_y);
		activeRectangle.setRect(new_x, new_y, activeRectangle.getWidth(), activeRectangle.getHeight());

		return false;
	}

	public void repaintAll() {
		repaint();
	}

	public void drawLine(Graphics2D g2, Line line, int perimeter, double radius) {
		g2.setColor(Color.BLUE);

		Ellipse2D.Double circle = new Ellipse2D.Double(line.getStart().x - radius, line.getStart().y - radius,
				perimeter, perimeter);
		g2.fill(circle);
		circle = new Ellipse2D.Double(line.getEnd().x - radius, line.getEnd().y - radius, perimeter, perimeter);
		g2.fill(circle);

		g2.drawLine((int) line.getStart().x, (int) line.getStart().y, (int) line.getEnd().x, (int) line.getEnd().y);
	}

	public void paint(Graphics g) {
		super.paint(g);
		
		setPositions(resizableManager.getPositions());
		
		// Draws the img to the BackgroundPanel.
		g.drawImage(image, 0, 0, null);
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		g2.setStroke(new BasicStroke(THICKNESS));

		g2.setColor(Color.BLUE);

		int perimeter = THICKNESS * 5;
		double radius = perimeter / 2;

		if (lineStartPoint != null && lineEndPoint != null) {
			Ellipse2D.Double circle = new Ellipse2D.Double(lineStartPoint.x - radius, lineStartPoint.y - radius,
					perimeter, perimeter);
			g2.fill(circle);
			circle = new Ellipse2D.Double(lineEndPoint.x - radius, lineEndPoint.y - radius, perimeter, perimeter);
			g2.fill(circle);

			g2.drawLine(lineStartPoint.x, lineStartPoint.y, lineEndPoint.x, lineEndPoint.y);
		}

		for (Line line : lines) {
			drawLine(g2, line, perimeter, radius);
		}

		if (activeResizable != null) {
			activeResizable.setRect(activeRectangle);
			drawRectangle(activeResizable, g);
		}

		for (Resizable region : positions) {
			if (region.isVisible() && !region.equals(activeResizable)) {
				drawRectangle(region, g);
			}
		}

		// System.out.println(validPositions.size() + " " +
		// fixedPositions.size());
	}

	private void drawRectangle(Resizable r, Graphics gr) {
		Graphics2D g = null;
		if (gr instanceof Graphics2D) {
			g = (Graphics2D) gr;
		} else {
			return;
		}
		g.setColor(r.getColor());

		if (isFirstTime) {
			area = new Rectangle(0, 0, image.getWidth(null), image.getHeight(null));
			isFirstTime = false;
		}

		g.draw(r.getRect());
		g.setPaint(r.getColor());
		g.fill(r.getRect());
	}

	public Image getImage() {
		return image;
	}

	public void setImage(Image image) {
		this.image = image;
	}

	public void updateImage(Mat image) {
		isFirstTime = true;
		setImageMat(image);

		BufferedImage inputBI = ImageProcessor.matToBufferedImage(image);
		setPreferredSize(new Dimension(inputBI.getWidth(null), inputBI.getHeight(null)));
		setSize(new Dimension(inputBI.getWidth(null), inputBI.getHeight(null)));
		this.image = inputBI;

		ArrayList<Resizable> newResizables = new ArrayList<Resizable>();
		
		if (positions != null) {
			for (Resizable resizable : positions) {
				if (resizable.isPermanent()) {
					newResizables.add(resizable);
				}
			}
		}

		setActiveResizable(null);
		setPositions(newResizables);
		
		repaint();
	}

	public Mat getImageMat() {
		return imageMat;
	}

	public void setImageMat(Mat imageMat) {
		this.imageMat = imageMat;
	}

	public BackgroundPanel getPnl() {
		return pnl;
	}

	public void setPnl(BackgroundPanel pnl) {
		this.pnl = pnl;
	}

	public Resizable getActiveResizable() {
		return activeResizable;
	}

	public void setActiveResizable(Resizable activeResizable) {
		this.activeResizable = activeResizable;

		if (activeResizable != null) {
			setActiveRectangle(activeResizable.getRect().getBounds());
		}
	}

	public Rectangle getActiveRectangle() {
		return activeRectangle;
	}

	public void setActiveRectangle(Rectangle activeRectangle) {
		activeRectangle.setRect(activeRectangle.getX(), activeRectangle.getY(), activeRectangle.getWidth(),
				activeRectangle.height);
		this.activeRectangle = activeRectangle;
	}

	public Resizable getProcessingResizable() {
		return processingResizable;
	}

	public void setProcessingResizable(Resizable processingResizable) {
		this.processingResizable = processingResizable;
	}

	public ArrayList<Line> getLines() {
		return lines;
	}

	public void setLines(ArrayList<Line> lines) {
		this.lines = lines;
	}

	public ArrayList<Resizable> getPositions() {
		return positions;
	}

	public void setPositions(ArrayList<Resizable> positions) {
		this.positions = positions;
	}

	public ResizableManager getResizableManager() {
		return resizableManager;
	}

	public void setResizableManager(ResizableManager resizableManager) {
		this.resizableManager = resizableManager;
	}	
}