package gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.prefs.Preferences;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JSpinner.DefaultEditor;
import javax.swing.SpinnerNumberModel;
import dataManagement.Page;
import net.miginfocom.swing.MigLayout;

@SuppressWarnings("serial")
public class GuiInit extends JFrame {

	private static final boolean LAYOUT_DEBUG = false;

	private static final int VERTICAL_RESOLUTION_MIN = 600;
	private static final int VERTICAL_RESOLUTION_MAX = 1200;

	private Preferences pref;

	private JPanel contentPane;

	private JButton btnSetInputFolder;
	private JButton btnSetOutputFolder;
	private JButton btnStartApplication;

	private JLabel lblInputFolder;
	private JLabel lblOutputFolder;
	private JLabel lblVertResText;
	private JLabel lblVertResPixels;

	private JSpinner spinVertRes;

	private JFileChooser fileChooserInput;
	private JFileChooser fileChooserOutput;

	private String inputPath;
	private String outputPath;

	public GuiInit() {
		super("Configuration");

		pref = Preferences.userRoot();

		createWidgits();
		addWidgets();
		addListener();

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		pack();
		setLocationRelativeTo(null);
		setMinimumSize(getSize());
	}

	private void startApplication() {
		pref.put("BIBSEG_DEFAULT_VERTICAL_RES", spinVertRes.getValue().toString());

		File inputFolder = new File(inputPath);
		File[] listOfFiles = inputFolder.listFiles();

		ArrayList<Page> pages = new ArrayList<Page>();

		for (int i = 0; i < listOfFiles.length; i++) {
			String filePath = listOfFiles[i].getAbsolutePath();
			Page page = null;

			if ((filePath.endsWith(".jpg")) || (filePath.endsWith(".jpeg")) || (filePath.endsWith(".tif"))
					|| (filePath.endsWith(".png"))) {
//				String identifier = "Page " + (i + 1);
				String identifier = filePath.substring(filePath.lastIndexOf(File.separator) + 1,
						filePath.lastIndexOf("."));

				page = new Page(filePath, identifier);

				pages.add(page);
			}
		}
		
		final ArrayList<Page> finalPages = pages;

		EventQueue.invokeLater(new Runnable() {
			public void run() {
				Gui gui = new Gui(finalPages, outputPath, (int) spinVertRes.getValue());
				gui.setResizable(false);
				gui.setVisible(true);
				dispose();
			}
		});
	}

	private void selectInputFolder() {
		// Retrieve the selected path or use
		// an empty string if no path has
		// previously been selected
		String inputPath = pref.get("BIBSEG_DEFAULT_INPUT_PATH", "Not set");

		// Set the path that was saved in preferences
		fileChooserInput.setCurrentDirectory(new File(inputPath));

		int returnVal = fileChooserInput.showOpenDialog(null);

		if (returnVal == JFileChooser.APPROVE_OPTION) {

			File selectedDirectory = fileChooserInput.getSelectedFile();
			inputPath = selectedDirectory.getAbsolutePath();
			setInputPath(inputPath);

			// Save the selected path
			pref.put("BIBSEG_DEFAULT_INPUT_PATH", selectedDirectory.getAbsolutePath());
		}
	}

	private void selectOutputFolder() {
		// Retrieve the selected path or use
		// an empty string if no path has
		// previously been selected
		String outputPath = pref.get("BIBSEG_DEFAULT_OUTPUT_PATH", "Not set");

		// Set the path that was saved in preferences
		fileChooserInput.setCurrentDirectory(new File(outputPath));

		int returnVal = fileChooserOutput.showOpenDialog(null);

		if (returnVal == JFileChooser.APPROVE_OPTION) {

			File selectedDirectory = fileChooserOutput.getSelectedFile();
			outputPath = selectedDirectory.getAbsolutePath();
			setOutputPath(outputPath);

			// Save the selected path
			pref.put("BIBSEG_DEFAULT_OUTPUT_PATH", selectedDirectory.getAbsolutePath());
		}
	}

	private void addListener() {
		btnSetInputFolder.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				selectInputFolder();
			}
		});

		btnSetOutputFolder.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				selectOutputFolder();
			}
		});

		btnStartApplication.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				startApplication();
			}
		});
	}

	private void addWidgets() {
		MigLayout layout = null;
		if (LAYOUT_DEBUG) {
			layout = new MigLayout("debug", "[][][]", "[][][]");
		} else {
			layout = new MigLayout("", "[][][]", "[][][]");
		}
		setLayout(layout);

		add(btnSetInputFolder, "left, sg 1, split");
		add(lblInputFolder, "span, pushx, growx, wrap");
		add(btnSetOutputFolder, "left, sg 1, split");
		add(lblOutputFolder, "span, pushx, growx, wrap");
		add(btnStartApplication, "split, left, sg 1");
		add(lblVertResText);
		add(spinVertRes);
		add(lblVertResPixels);
	}

	private void createWidgits() {
		contentPane = new JPanel();
		setContentPane(contentPane);

		btnSetInputFolder = new JButton("Set Input Folder");
		btnSetOutputFolder = new JButton("Set Output Folder");
		btnStartApplication = new JButton("Start Application");
		btnStartApplication.setEnabled(false);

		String inputPath = pref.get("BIBSEG_DEFAULT_INPUT_PATH", "Not set");

		lblInputFolder = new JLabel();
		lblInputFolder.setMinimumSize(new Dimension(300, lblInputFolder.getSize().height));
		lblInputFolder.setBackground(Color.WHITE);
		lblInputFolder.setOpaque(true);
		lblInputFolder.setBorder(BorderFactory.createLineBorder(Color.black));
		setInputPath(inputPath);

		String outputPath = pref.get("BIBSEG_DEFAULT_OUTPUT_PATH", "Not set");

		lblOutputFolder = new JLabel();
		lblOutputFolder.setBackground(Color.WHITE);
		lblOutputFolder.setOpaque(true);
		lblOutputFolder.setBorder(BorderFactory.createLineBorder(Color.black));
		setOutputPath(outputPath);

		lblVertResText = new JLabel("Vertical Image Resolution:");
		lblVertResPixels = new JLabel("Pixels");

		int verticalRes = Integer.parseInt(pref.get("BIBSEG_DEFAULT_VERTICAL_RES", "1000"));

		spinVertRes = new JSpinner(new SpinnerNumberModel(verticalRes, VERTICAL_RESOLUTION_MIN, VERTICAL_RESOLUTION_MAX, 50));
		spinVertRes.setMaximumSize(new Dimension(50, 20));
		JSpinner.NumberEditor editor = new JSpinner.NumberEditor(spinVertRes, "#");
		spinVertRes.setEditor(editor);
		((DefaultEditor) spinVertRes.getEditor()).getTextField().setEditable(false);

		fileChooserInput = new JFileChooser(System.getProperty("user.home") + File.separator + "Desktop");
		fileChooserInput.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

		fileChooserOutput = new JFileChooser(System.getProperty("user.home") + File.separator + "Desktop");
		fileChooserOutput.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	}

	public String getInputPath() {
		return inputPath;
	}

	public void setInputPath(String inputPath) {
		lblInputFolder.setText(inputPath);

		if (outputPath != null) {
			btnStartApplication.setEnabled(true);
		}

		if (!inputPath.endsWith(File.separator)) {
			inputPath += File.separator;
		}

		this.inputPath = inputPath;
	}

	public String getOutputPath() {
		return outputPath;
	}

	public void setOutputPath(String outputPath) {
		lblOutputFolder.setText(outputPath);

		if (inputPath != null) {
			btnStartApplication.setEnabled(true);
		}

		if (!outputPath.endsWith(File.separator)) {
			outputPath += File.separator;
		}

		this.outputPath = outputPath;
	}
}