package gui;

import java.io.File;
import org.opencv.core.Mat;
import org.opencv.core.Size;

import dataManagement.DataManager;
import dataManagement.Page;
import export.PageXMLWriter;
import regions.RegionManager;
import regions.ResizableManager;
import segmentation.ImageProcessor;
import segmentation.SegmentationResult;
import segmentation.Segmenter;
import segmentation.parameters.Parameters;

public class GuiManager implements Runnable {

	private boolean keepSegmenting;

	private String outputFolder;

	private Gui gui;
	private DataManager dataManager;
	private RegionManager regionManager;
	private ResizableManager resizableManager;
	private Page currentPage;

	private Thread runner;

	public GuiManager(Gui gui) {
		setGui(gui);
		setDataManager(gui.getDataManager());
		setRegionManager(new RegionManager(gui));
		regionManager.initRegions(dataManager.getPages().get(0).getResized());

		setCurrentPage(gui.getDataManager().getCurrentPage());
		setResizableManager(new ResizableManager(gui, regionManager));
	}

	public void saveCurrentPage() {
		PageXMLWriter.writePageXML(currentPage, outputFolder);
	}

	public void segmentAllPages() {
		setKeepSegmenting(true);
		gui.getPnlPages().getProcessedPages().removeAll(gui.getPnlPages().getProcessedPages());
		gui.getPnlPages().getListPages().repaint();

		segmentCurrentPage();

		startSegmentation();
	}

	public void segmentCurrentPage() {
		Mat original = currentPage.getOriginal();
		Parameters parameters = new Parameters(resizableManager, gui);
		currentPage.setScaleFactor((double) currentPage.getOriginal().height() / parameters.getDesiredImageHeight());

		SegmentationResult segmentationResult = Segmenter.segment(original, parameters);
		currentPage.setSegmentationResult(segmentationResult);

		currentPage.visualizeSegmentation();
		gui.showImage(currentPage.getResult());
	}

	public void showNextPage() {
		int index = dataManager.getIndex();

		if (index < dataManager.getPages().size() - 1) {
			index++;
			gui.getPnlPages().getListPages().setSelectedIndex(index);
			loadPage(dataManager.getPages().get(index).getFileName());
		}
	}

	public void showPreviousPage() {
		int index = dataManager.getIndex();

		if (index > 0) {
			index--;
			gui.getPnlPages().getListPages().setSelectedIndex(index);
			loadPage(dataManager.getPages().get(index).getFileName());
		}
	}

	public Page identifyPage(String fileName) {
		Page page = null;

		for (int i = 0; i < dataManager.getPages().size(); i++) {
			if (dataManager.getPages().get(i).getIdentifier().equals(fileName)) {
				page = dataManager.getPages().get(i);
				break;
			}
		}
		
		return page;
	}
	
	public void loadPage(String fileName) {
		Page page = identifyPage(fileName);
		loadPage(page);
	}
	
	public void loadPage(Page page) {
//		Size oldSize = currentPage.getResized().size();
		
		currentPage.clean();

//		Page page = identifyPage(fileName);

		page.initPage(gui.getVerticalResolution(), resizableManager);

		dataManager.setIndex(gui.getPnlPages().getListPages().getSelectedIndex());
		dataManager.setCurrentPage(page);
		setCurrentPage(gui.getDataManager().getCurrentPage());
		
//		Size newSize = currentPage.getResized().size();
		
//		System.out.print(resizableManager.getPositions().get(0).getRectOCV() + " ");
//		resizableManager.update(oldSize, newSize);
//		System.out.print(resizableManager.getPositions().get(0).getRectOCV() + " ");
		
		gui.getPnlBackground().updateImage(page.getResized());
		gui.pack();
		gui.revalidate();
		gui.repaint();
		
		if (currentPage.getSegmentationResult() != null) {
			gui.getRbResult().setEnabled(true);
		}

		if (gui.getRbResult().isSelected()) {
			if (currentPage.getResult() != null) {
				gui.showImage(currentPage.getResult());
			} else {
				gui.showImage(currentPage.getResized());
				gui.getRbOriginal().setSelected(true);
				gui.getRbBinary().setSelected(false);
				gui.getRbResult().setSelected(false);
				gui.getRbResult().setEnabled(false);
			}
		} else if (gui.getRbOriginal().isSelected()) {
			gui.showImage(currentPage.getResized());
		} else if (gui.getRbBinary().isSelected()) {
			if (currentPage.getBinary() != null) {
				gui.showImage(currentPage.getBinary());
			} else {
				calcAndShowBinary();
			}
		}

		if (currentPage.getResult() != null) {
			gui.getRbResult().setEnabled(true);
		} else {
			gui.getRbResult().setEnabled(false);
		}
		
		resizableManager.getLines().removeAll(resizableManager.getLines());
	}

	public void calcAndShowBinary() {
		int binaryThresh = (int) gui.getPnlProcessParameters().getSpinBinaryThresh().getValue();

		Mat gray = ImageProcessor.calcGray(currentPage.getOriginal());
		Mat binary = new Mat();

		if (binaryThresh == -1) {
			binary = ImageProcessor.calcBinary(gray);
		} else {
			binary = ImageProcessor.calcBinaryFromThresh(gray, binaryThresh);
		}

		currentPage.setBinary(binary);
		gui.showImage(binary);
	}

	public Gui getGui() {
		return gui;
	}

	public void setGui(Gui gui) {
		this.gui = gui;
	}

	public DataManager getDataManager() {
		return dataManager;
	}

	public void setDataManager(DataManager dataManager) {
		this.dataManager = dataManager;
	}

	public Page getCurrentPage() {
		return currentPage;
	}

	public void setCurrentPage(Page currentPage) {
		this.currentPage = currentPage;
	}

	public void startSegmentation() {
		// System.out.println("start seg");
		// if (runner == null) {
		// System.out.println("runner = null");
		// runner = new Thread(this);
		// runner.start();
		// }

		runner = new Thread(this);
		runner.start();
	}

	public void stop() {
		if (runner != null) {
			runner.interrupt();
			runner = null;
			// runner.stop();
		}
	}

	public void run() {
		for (int i = gui.getPnlPages().getListPages().getSelectedIndex(); i < dataManager.getPages().size(); i++) {
			if (keepSegmenting) {
				Page page = dataManager.getPages().get(i);

				if (!page.isAccepted()) {
					page.initPage(gui.getVerticalResolution(), resizableManager);

					Parameters parameters = new Parameters(resizableManager, gui);
					page.setScaleFactor((double) page.getOriginal().height() / parameters.getDesiredImageHeight());

					SegmentationResult segmentationResult = Segmenter.segment(page.getOriginal(), parameters);
					page.setSegmentationResult(segmentationResult);

					gui.getPnlPages().getProcessedPages().add(i);
					gui.getPnlPages().getListPages().repaint();
				}
			}
			if (i > 0) {
				dataManager.getPages().get(i - 1).clean();
			}
		}
	}

	public boolean isKeepSegmenting() {
		return keepSegmenting;
	}

	public String getOutputFolder() {
		return outputFolder;
	}

	public void setOutputFolder(String outputFolder) {
		if (!outputFolder.endsWith(File.separator)) {
			outputFolder += File.separator;
		}
		
		this.outputFolder = outputFolder;
	}

	public void setKeepSegmenting(boolean keepSegmenting) {
		this.keepSegmenting = keepSegmenting;
	}

	public RegionManager getRegionManager() {
		return regionManager;
	}

	public void setRegionManager(RegionManager regionManager) {
		this.regionManager = regionManager;
	}

	public ResizableManager getResizableManager() {
		return resizableManager;
	}

	public void setResizableManager(ResizableManager resizableManager) {
		this.resizableManager = resizableManager;
	}
}