package regions;

import java.awt.Color;
import java.util.ArrayList;

import org.opencv.core.Mat;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;

import gui.Gui;
import regions.type.RegionType;
import segmentation.parameters.DEFAULT_Parameters;

public class RegionManager {

	private Gui gui;
	private  ArrayList<Region> regions;
	private  ArrayList<Resizable> resizables;
	private String[] usedRegionTypes;

	public RegionManager(Gui gui) {
		setGui(gui);

		String[] usedRegionTypes = { "Image", "Paragraph", "Marginalia", "PageNumber" };
		setUsedRegionTypes(usedRegionTypes);
	}

	public void initRegions(Mat input) {
		ArrayList<Region> regions = new ArrayList<Region>();

		Region image = new Region(RegionType.IMAGE, "image", DEFAULT_Parameters.getImageMinSizeDefault(), -1,
				new Color(0, 255, 0, 50), new Scalar(0, 255, 0));
		Rect imageRect = new Rect(new Point(0, 0), new Point(input.width() - 1, input.height() - 1));
		image.setPositions(imageRect);
		regions.add(image);

		Region text = new Region(RegionType.PARAGRAPH, "paragraph", DEFAULT_Parameters.getParagraphMinSizeDefault(), -1,
				new Color(255, 0, 0, 50), new Scalar(0, 0, 255));
		Rect textRect = new Rect(new Point(0, 0), new Point(input.width() - 1, input.height() - 1));
		text.setPositions(textRect);
		regions.add(text);

		Region marginalia = new Region(RegionType.MARGINALIA, "marginalia",
				DEFAULT_Parameters.getMarginaliaMinSizeDefault(), -1, new Color(255, 255, 0, 50),
				new Scalar(0, 255, 255));
		marginalia.setPositions(calcMarginaliaPosition(input, DEFAULT_Parameters.getMarginaliaLeftRightPercentage()));
		regions.add(marginalia);

		Region pageNumber = new Region(RegionType.PAGE_NUMBER, "page-number",
				DEFAULT_Parameters.getPageNumberMinSizeDefault(), -1, new Color(255, 0, 255, 50),
				new Scalar(255, 0, 255));
		pageNumber.setPositions(calcPageNumberPosition(input, DEFAULT_Parameters.getMarginaliaLeftRightPercentage()));
		regions.add(pageNumber);

		setRegions(regions);
		calcResizables();
	}

	public ArrayList<Rect> calcMarginaliaPosition(Mat input, int leftRightPercentage) {
		ArrayList<Rect> positions = new ArrayList<Rect>();

		Rect left = new Rect(new Point(0, 0), new Point(leftRightPercentage * input.width() / 100, input.height() - 1));
		Rect right = new Rect(new Point((100 - leftRightPercentage) * input.width() / 100, 0),
				new Point(input.width() - 1, input.height() - 1));

		positions.add(left);
		positions.add(right);

		return positions;
	}

	public ArrayList<Rect> calcPageNumberPosition(Mat input, int topBottomPercentage) {
		ArrayList<Rect> positions = new ArrayList<Rect>();

		Rect top = new Rect(new Point(0, 0), new Point(input.width() - 1, topBottomPercentage * input.height() / 100));
		Rect bottom = new Rect(new Point(0, (100 - topBottomPercentage) * input.height() / 100),
				new Point(input.width() - 1, input.height() - 1));

		positions.add(top);
		positions.add(bottom);

		return positions;
	}

	public void calcResizables() {
		ArrayList<Resizable> resizables = new ArrayList<Resizable>();
		
		for(Region region : regions) {
			for(Rect position : region.getPositions()) {
				Resizable resizable = new Resizable(position, region.getType());
				resizables.add(resizable);
			}
		}
		
		setResizables(resizables);
	}
	
	public Gui getGui() {
		return gui;
	}

	public void setGui(Gui gui) {
		this.gui = gui;
	}

	public String[] getUsedRegionTypes() {
		return usedRegionTypes;
	}

	public void setUsedRegionTypes(String[] usedRegionTypes) {
		this.usedRegionTypes = usedRegionTypes;
	}

	public ArrayList<Region> getRegions() {
		return regions;
	}

	public void setRegions(ArrayList<Region> regions) {
		this.regions = regions;
	}

	public ArrayList<Resizable> getResizables() {
		return resizables;
	}

	public void setResizables(ArrayList<Resizable> resizables) {
		this.resizables = resizables;
	}	
}