package regions;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import org.opencv.core.Rect;
import org.opencv.core.Size;

import gui.BackgroundPanel;
import gui.Gui;
import gui.initParameters.PnlRegionParameters;
import regions.type.RegionType;

public class ResizableManager {

	private static final int PROX_DIST = 3;

	private Gui gui;
	private BackgroundPanel pnlBackground;
	private RegionManager regionManager;

	private ArrayList<Resizable> positions;
	private ArrayList<Resizable> sizes;
	private ArrayList<Line> lines;

	private Size lastSize;

	public ResizableManager(Gui gui, RegionManager regionManager) {
		setGui(gui);
		setPnlBackground(gui.getPnlBackground());
		setRegionManager(regionManager);
		init();
	}

	public void update(Size next) {
		double scaleFactor = (double) next.width / lastSize.width;
		ArrayList<Resizable> removed = new ArrayList<Resizable>();

		for (Resizable position : positions) {
			if(!position.isPermanent()) {
				removed.add(position);
			} else {
				Rect oldRect = position.getRectOCV();

				Rectangle2D newRect = new Rectangle2D.Double(scaleFactor * oldRect.x, oldRect.y,
						scaleFactor * oldRect.width, oldRect.height);
				position.setRect(newRect);
			}
		}
		
		positions.removeAll(removed);
		setLastSize(next);
	}

	public void init() {
		setPositions(new ArrayList<Resizable>());
		setLines(new ArrayList<Line>());

		for (Region region : regionManager.getRegions()) {
			for (Rect rect : region.getPositions()) {
				Resizable resizable = new Resizable(rect, region.getType());
				// gui.getLayeredPane().add(resizable, 2);
				// resizable.setVisible(false);
				resizable.setPermanent(true);
				positions.add(resizable);
			}
		}
	}

	public void showPositions(RegionType regionType, boolean show) {
		for (Resizable resizable : positions) {
			if (resizable.getType().equals(regionType)) {
				if (!resizable.isFixed()) {
					resizable.setVisible(show);
				}
			}
		}

		pnlBackground.setActiveResizable(null);
		pnlBackground.repaintAll();
	}

	public void showFixedPositions(boolean show) {
		for (Resizable resizable : positions) {
			if (resizable.isFixed()) {
				resizable.setVisible(show);
			}
		}

		pnlBackground.setActiveResizable(null);
		pnlBackground.repaintAll();
	}

	public void showRegionSizes(boolean show) {
		if (sizes == null) {
			setSizes(new ArrayList<Resizable>());

			PnlRegionParameters pnlRegionParameters = gui.getPnlRegionParameters();

			int imageSize = (int) pnlRegionParameters.getSpinImageMinSize().getValue();
			int paragraphSize = (int) pnlRegionParameters.getSpinTextMinSize().getValue();
			int marginaliaSize = (int) pnlRegionParameters.getSpinMarginaliaMinSize().getValue();
			int pageNumberSize = (int) pnlRegionParameters.getSpinPageNumberMinSize().getValue();

			int imageWidth = (int) Math.sqrt(imageSize);
			int paragraphWidth = (int) Math.sqrt(paragraphSize);
			int marginaliaWidth = (int) Math.sqrt(marginaliaSize);
			int pageNumberWidth = (int) Math.sqrt(pageNumberSize);

			Resizable image = new Resizable(new Rect(0, 0, imageWidth, imageWidth), RegionType.IMAGE);
			Resizable paragraph = new Resizable(new Rect(0, 0, paragraphWidth, paragraphWidth), RegionType.PARAGRAPH);
			Resizable marginalia = new Resizable(new Rect(0, 0, marginaliaWidth, marginaliaWidth),
					RegionType.MARGINALIA);
			Resizable pageNumber = new Resizable(new Rect(0, 0, pageNumberWidth, pageNumberWidth),
					RegionType.PAGE_NUMBER);

			sizes.add(image);
			sizes.add(paragraph);
			sizes.add(marginalia);
			sizes.add(pageNumber);

			pnlBackground.repaintAll();
		}

		// for (Resizable reziable : sizes) {
		// reziable.setVisible(show);
		// }
	}

	private static Resizable findSmallestRect(ArrayList<Resizable> regions) {
		Resizable result = null;
		double minArea = Double.MAX_VALUE;

		for (Resizable temp : regions) {
			double area = temp.getRectOCV().area();
			if (area < minArea) {
				minArea = area;
				result = temp;
			}
		}

		return result;
	}

	public static Resizable identifyActiveRect(MouseEvent e, ArrayList<Resizable> positions) {
		Point point = new Point(e.getX(), e.getY());

		ArrayList<Resizable> hits = new ArrayList<Resizable>();

		for (Resizable region : positions) {
			if (region.isVisible() && isOverRect(point, region.getRect().getBounds())) {
				manageActive(region, positions);
				hits.add(region);
			}
		}

		if (hits.size() == 0) {
			return null;
		} else if (hits.size() == 1) {
			return hits.get(0);
		} else {
			Resizable result = findSmallestRect(hits);
			manageActive(result, positions);
			return result;
		}
	}

	private static boolean isOverRect(Point p, Rectangle rect) {
		Rectangle r = (Rectangle) rect.clone();
		r.grow(PROX_DIST, PROX_DIST);

		return r.contains(p);
	}

	public static void manageActive(Resizable region, ArrayList<Resizable> positions) {
		for (Resizable tempRegion : positions) {
			if (tempRegion.equals(region)) {
				tempRegion.setActive(true);
			} else {
				tempRegion.setActive(false);
			}
		}
	}

	public Gui getGui() {
		return gui;
	}

	public void setGui(Gui gui) {
		this.gui = gui;
	}

	public BackgroundPanel getPnlBackground() {
		return pnlBackground;
	}

	public void setPnlBackground(BackgroundPanel pnlBackground) {
		this.pnlBackground = pnlBackground;
	}

	public RegionManager getRegionManager() {
		return regionManager;
	}

	public void setRegionManager(RegionManager regionManager) {
		this.regionManager = regionManager;
	}

	public ArrayList<Resizable> getPositions() {
		return positions;
	}

	public void setPositions(ArrayList<Resizable> positions) {
		this.positions = positions;
	}

	public ArrayList<Resizable> getSizes() {
		return sizes;
	}

	public void setSizes(ArrayList<Resizable> sizes) {
		this.sizes = sizes;
	}

	public ArrayList<Line> getLines() {
		return lines;
	}

	public void setLines(ArrayList<Line> lines) {
		this.lines = lines;
	}

	public Size getLastSize() {
		return lastSize;
	}

	public void setLastSize(Size lastSize) {
		this.lastSize = lastSize;
	}

	public ArrayList<Resizable> calcTruePositions(double scaleFactor) {
		ArrayList<Resizable> truePositions = new ArrayList<Resizable>();

		for (Resizable position : positions) {
			Rect oldRect = position.getRectOCV();
			Rect rect = new Rect((int) (scaleFactor * oldRect.x), (int) (scaleFactor * oldRect.y),
					(int) (scaleFactor * oldRect.width), (int) (scaleFactor * oldRect.height));
			Resizable newPosition = new Resizable(rect, position.getType());
			truePositions.add(newPosition);
			// System.out.println(rect);
		}

		return truePositions;
	}
}
