package segmentation;

import java.util.ArrayList;

import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.Rect;
import org.opencv.imgproc.Imgproc;

import regions.Resizable;
import regions.type.RegionType;
import segmentation.parameters.Parameters;
import segmentation.verfication.MarginaliaVerifier;
import segmentation.verfication.PageNumberVerifier;

public class RegionClassifier {

	private static final int MAX_TEXT_ASPECT_RATIO = 50;

	private Mat binary;
	private Parameters parameters;
	
	private ArrayList<Resizable> truePositions;

	private MarginaliaVerifier marginaliaVerifier;
	private PageNumberVerifier pageNumberVerifier;

	private int minPictureArea;
	private int minTextArea;
	private int minMarginaliaArea;
	private int minPageNumberArea;

	private boolean marginaliaPresent;
	private boolean pageNumberPresent;
	private boolean pageNumberTop;
	private boolean pageNumberBottom;
	private boolean captionBelow;

	public RegionClassifier(Mat binary, Parameters parameters) {
		setBinary(binary);
		setParameters(parameters);
		
		setMinPictureArea(parameters.getMinImageSize());
		setMinTextArea(parameters.getMinTextSize());
		setMinMarginaliaArea(parameters.getMinMarginaliaSize());
		setMinPageNumberArea(parameters.getMinPageNumberSize());

		setMarginaliaPresent(parameters.isMarginaliaPresent());
		setPageNumberTop(parameters.isPageNumberTop());
		setPageNumberBottom(parameters.isPageNumberBottom());
		setCaptionBelow(parameters.isCaptionBelow());

		if(pageNumberTop || pageNumberBottom) {
			setPageNumberPresent(true);
		}
		
		setMarginaliaVerifier(
				new MarginaliaVerifier(binary, parameters));
		setPageNumberVerifier(new PageNumberVerifier(binary, parameters));
	}

	public boolean positionIsValid(Rect rect, RegionType type) {
		boolean isWithin = false;
		
		for(Resizable region : truePositions) {
//			System.out.println(region.getRectOCV());
			if(region.getType().equals(type) && region.getRectOCV().contains(rect.tl()) && region.getRectOCV().contains(rect.br())) {
				isWithin = true;
				break;
			}
		}
		
		return isWithin;
	}
	
	public ArrayList<ArrayList<MatOfPoint>> findTextAndMarginaliaContoursNew(ArrayList<MatOfPoint> contours) {
		ArrayList<ArrayList<MatOfPoint>> textAndMarginalia = new ArrayList<ArrayList<MatOfPoint>>();
		ArrayList<MatOfPoint> textContours = new ArrayList<MatOfPoint>();
		ArrayList<MatOfPoint> marginaliaContours = new ArrayList<MatOfPoint>();
		ArrayList<MatOfPoint> pageNumberContours = new ArrayList<MatOfPoint>();
		
		truePositions = parameters.getResizableManager().calcTruePositions(parameters.getScaleFactor());
		
		int minArea = Math.min(Math.min(minPageNumberArea, minMarginaliaArea), minTextArea);
		
		for (MatOfPoint contour : contours) {
			Rect rect = Imgproc.boundingRect(contour);
			double area = rect.area();
			
			if (area > minArea) {
				if(marginaliaPresent && area > minMarginaliaArea && positionIsValid(rect, RegionType.MARGINALIA)) {
//					System.out.println("marg");
					marginaliaContours.add(contour);
				} else if(pageNumberPresent && area > minPageNumberArea && positionIsValid(rect, RegionType.PAGE_NUMBER)) {
//					System.out.println("pn");
					pageNumberContours.add(contour);
//				} else if(area > minTextArea && positionIsValid(rect, RegionType.PARAGRAPH)) {
				} else if(area > minTextArea) {
//					System.out.println("text");
					textContours.add(contour);
				}
			}
		}
		
		if (pageNumberContours.size() > 1) {
			pageNumberVerifier.identifyPageNumber(textContours, pageNumberContours, minTextArea);
		}

		textAndMarginalia.add(textContours);
		textAndMarginalia.add(marginaliaContours);
		textAndMarginalia.add(pageNumberContours);
		
		return textAndMarginalia;
	}
		
//	public ArrayList<ArrayList<MatOfPoint>> findTextAndMarginaliaContours(ArrayList<MatOfPoint> contours) {
//		ArrayList<ArrayList<MatOfPoint>> textAndMarginalia = new ArrayList<ArrayList<MatOfPoint>>();
//		ArrayList<MatOfPoint> textContours = new ArrayList<MatOfPoint>();
//		ArrayList<MatOfPoint> marginaliaContours = new ArrayList<MatOfPoint>();
//		ArrayList<MatOfPoint> pageNumberContours = new ArrayList<MatOfPoint>();
//
//		int minArea = Math.min(Math.min(minPageNumberArea, minMarginaliaArea), minTextArea);
//
//		for (MatOfPoint contour : contours) {
//			Rect rect = Imgproc.boundingRect(contour);
//			if (rect.area() > minArea) {
//				if (marginaliaPresent && rect.area() > minMarginaliaArea
//						&& marginaliaVerifier.checkMarginaliaPosition(rect)
//						&& checkAspectRatio(rect, MAX_TEXT_ASPECT_RATIO)) {
//					marginaliaContours.add(contour);
//				} else if (pageNumberPresent && rect.area() > minPageNumberArea && pageNumberVerifier.checkPageNumberPosition(rect)) {
//					pageNumberContours.add(contour);
//				} else if (rect.area() > minTextArea && checkAspectRatio(rect, MAX_TEXT_ASPECT_RATIO)) {
//					textContours.add(contour);
//				}
//			}
//		}
//
//		if (pageNumberContours.size() > 1) {
//			pageNumberVerifier.identifyPageNumber(textContours, pageNumberContours);
//		}
//
//		if (marginaliaContours.size() > 0) {
////			marginaliaVerifier.verifyMarginalia(textContours, marginaliaContours);
//		}
//
//		textAndMarginalia.add(textContours);
//		textAndMarginalia.add(marginaliaContours);
//		textAndMarginalia.add(pageNumberContours);
//
//		return textAndMarginalia;
//	}

	private boolean checkAspectRatio(Rect rect, int maxRatio) {
		int longer = 0;
		int shorter = 0;

		if (rect.width > rect.height) {
			longer = rect.width;
			shorter = rect.height;
		} else {
			longer = rect.height;
			shorter = rect.width;
		}

		if ((double) longer / shorter > maxRatio) {
			return false;
		}
		return true;
	}

	public Mat getBinary() {
		return binary;
	}

	public void setBinary(Mat binary) {
		this.binary = binary;
	}

	public Parameters getParameters() {
		return parameters;
	}

	public void setParameters(Parameters parameters) {
		this.parameters = parameters;
	}

	public MarginaliaVerifier getMarginaliaVerifier() {
		return marginaliaVerifier;
	}

	public void setMarginaliaVerifier(MarginaliaVerifier marginaliaVerifier) {
		this.marginaliaVerifier = marginaliaVerifier;
	}

	public PageNumberVerifier getPageNumberVerifier() {
		return pageNumberVerifier;
	}

	public void setPageNumberVerifier(PageNumberVerifier pageNumberVerifier) {
		this.pageNumberVerifier = pageNumberVerifier;
	}

	public int getMinPictureArea() {
		return minPictureArea;
	}

	public void setMinPictureArea(int minPictureArea) {
		this.minPictureArea = minPictureArea;
	}

	public int getMinTextArea() {
		return minTextArea;
	}

	public void setMinTextArea(int minTextArea) {
		this.minTextArea = minTextArea;
	}

	public int getMinMarginaliaArea() {
		return minMarginaliaArea;
	}

	public void setMinMarginaliaArea(int minMarginaliaArea) {
		this.minMarginaliaArea = minMarginaliaArea;
	}

	public int getMinPageNumberArea() {
		return minPageNumberArea;
	}

	public void setMinPageNumberArea(int minPageNumberArea) {
		this.minPageNumberArea = minPageNumberArea;
	}

	public boolean isMarginaliaPresent() {
		return marginaliaPresent;
	}

	public void setMarginaliaPresent(boolean marginaliaPresent) {
		this.marginaliaPresent = marginaliaPresent;
	}

	public boolean isPageNumberPresent() {
		return pageNumberPresent;
	}

	public void setPageNumberPresent(boolean pageNumberPresent) {
		this.pageNumberPresent = pageNumberPresent;
	}

	public boolean isPageNumberTop() {
		return pageNumberTop;
	}

	public void setPageNumberTop(boolean pageNumberTop) {
		this.pageNumberTop = pageNumberTop;
	}

	public boolean isPageNumberBottom() {
		return pageNumberBottom;
	}

	public void setPageNumberBottom(boolean pageNumberBottom) {
		this.pageNumberBottom = pageNumberBottom;
	}

	public boolean isCaptionBelow() {
		return captionBelow;
	}

	public void setCaptionBelow(boolean captionBelow) {
		this.captionBelow = captionBelow;
	}
}