package segmentation.verfication;

import java.util.ArrayList;

import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.core.MatOfPoint;
import org.opencv.core.Point;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.imgproc.Imgproc;

import imageProcessing.Contour;
import segmentation.parameters.Parameters;

public class MarginaliaVerifier {

	private static final double MARGINALIA_HORIZONTAL_POSITION_PERCENTAGE = 0.25;
	
	private Mat binary;
	private int minMarginaliaArea;
	private double horizontalMarginaliaPosition;

	private boolean leftSidedMarginalia = false;
	private boolean cutMarginalia;

	public MarginaliaVerifier(Mat binary, Parameters parameters) {
		setBinary(binary);
		setMinMarginaliaArea(minMarginaliaArea);
		setCutMarginalia(parameters.isCutMarginalia());
		setHorizontalMarginaliaPosition(MARGINALIA_HORIZONTAL_POSITION_PERCENTAGE);
	}

	public void checkBorderCrossingMarginalia(ArrayList<MatOfPoint> textContours,
			ArrayList<MatOfPoint> marginaliaContours, int border) {
		ArrayList<MatOfPoint> acceptedMarginalia = new ArrayList<MatOfPoint>();

		for (MatOfPoint temp : marginaliaContours) {
			Rect rect = Imgproc.boundingRect(temp);

			if (rect.tl().x < border && rect.br().x > border) {
				int left = border - (int) rect.tl().x;
				int right = (int) rect.br().x - border;

				if (leftSidedMarginalia) {
					if (right > left) {
						textContours.add(temp);
					} else {
						textContours.remove(temp);
						acceptedMarginalia.add(temp);
					}
				} else {
					if (right < left) {
						textContours.add(temp);
					} else {
						textContours.remove(temp);
						acceptedMarginalia.add(temp);
					}
				}
			} else {
				acceptedMarginalia.add(temp);
			}
		}

		marginaliaContours.removeAll(marginaliaContours);
		marginaliaContours.addAll(acceptedMarginalia);
	}

	public void checkVerticallyForMoreMarginalia(ArrayList<MatOfPoint> textContours,
			ArrayList<MatOfPoint> marginaliaContours) {
		int sum = 0;
		int weight = 0;

		for (MatOfPoint temp : marginaliaContours) {
			Rect rect = Imgproc.boundingRect(temp);
			double area = Imgproc.contourArea(temp);

			if (leftSidedMarginalia) {
				sum += rect.br().x * area;
				weight += area;
			} else {
				sum += rect.tl().x * area;
				weight += area;
			}
		}

		int border = sum / weight;

		if(leftSidedMarginalia) {
			border -= 2;
		} else {
			border += 2;
		}
		
//		checkBorderCrossingMarginalia(textContours, marginaliaContours, border);

		Mat tempBinary = new Mat(binary.size(), binary.type(), new Scalar(0));
		Imgproc.drawContours(tempBinary, textContours, -1, new Scalar(255), -1);
		
		if(cutMarginalia) {
			Core.line(tempBinary, new Point(border, 0), new Point(border, binary.height() - 1), new Scalar(0));
		}

		ArrayList<MatOfPoint> contours = Contour.findContours(tempBinary);
		ArrayList<MatOfPoint> newMarginalia = new ArrayList<MatOfPoint>();
		ArrayList<MatOfPoint> newText = new ArrayList<MatOfPoint>();

		for (MatOfPoint temp : contours) {
			Rect rect = Imgproc.boundingRect(temp);

			if (Imgproc.contourArea(temp) > minMarginaliaArea && rect.width > 20) {
				if ((leftSidedMarginalia && rect.br().x <= border) || (!leftSidedMarginalia && rect.tl().x >= border)) {
					newMarginalia.add(temp);
				} else {
					newText.add(temp);
				}
			}
		}

		textContours.removeAll(textContours);
		textContours.addAll(newText);
		marginaliaContours.addAll(newMarginalia);
	}

	public void forceOneSidedMarginalia(ArrayList<MatOfPoint> textContours, ArrayList<MatOfPoint> marginaliaContours) {
		int border = binary.width() / 2;

		ArrayList<MatOfPoint> rightMarginalia = new ArrayList<MatOfPoint>();
		ArrayList<MatOfPoint> leftMarginalia = new ArrayList<MatOfPoint>();

		int leftCnt = 0;
		int rightCnt = 0;

		for (MatOfPoint temp : marginaliaContours) {
			Rect rect = Imgproc.boundingRect(temp);

			if (rect.tl().x > border) {
				rightCnt++;
				rightMarginalia.add(temp);
			} else {
				leftCnt++;
				leftMarginalia.add(temp);
			}
		}

		if (leftCnt > rightCnt) {
			leftSidedMarginalia = true;
			marginaliaContours.removeAll(marginaliaContours);
			marginaliaContours.addAll(leftMarginalia);
			textContours.addAll(rightMarginalia);
		} else if (leftCnt < rightCnt) {
			marginaliaContours.removeAll(marginaliaContours);
			marginaliaContours.addAll(rightMarginalia);
			textContours.addAll(leftMarginalia);
		}
	}

	public void verifyMarginalia(ArrayList<MatOfPoint> textContours, ArrayList<MatOfPoint> marginaliaContours) {
//		System.out.println("text: " + textContours.size() + " marg: " + marginaliaContours.size());
		forceOneSidedMarginalia(textContours, marginaliaContours);
//		System.out.println("text: " + textContours.size() + " marg: " + marginaliaContours.size());
		checkVerticallyForMoreMarginalia(textContours, marginaliaContours);
	}

	public boolean checkMarginaliaPosition(Rect rect) {
		int left = (int) (binary.width() * horizontalMarginaliaPosition);
		int right = binary.width() - (int) (binary.width() * horizontalMarginaliaPosition);

		if (rect.br().x <= left || rect.x >= right) {
			return true;
		} else {
			return false;
		}
	}

	public Mat getBinary() {
		return binary;
	}

	public void setBinary(Mat binary) {
		this.binary = binary;
	}

	public int getMinMarginaliaArea() {
		return minMarginaliaArea;
	}

	public void setMinMarginaliaArea(int minMarginaliaArea) {
		this.minMarginaliaArea = minMarginaliaArea;
	}

	public double getHorizontalMarginaliaPosition() {
		return horizontalMarginaliaPosition;
	}

	public void setHorizontalMarginaliaPosition(double horizontalMarginaliaPosition) {
		this.horizontalMarginaliaPosition = horizontalMarginaliaPosition;
	}

	public boolean isLeftSidedMarginalia() {
		return leftSidedMarginalia;
	}

	public void setLeftSidedMarginalia(boolean leftSidedMarginalia) {
		this.leftSidedMarginalia = leftSidedMarginalia;
	}

	public boolean isCutMarginalia() {
		return cutMarginalia;
	}

	public void setCutMarginalia(boolean cutMarginalia) {
		this.cutMarginalia = cutMarginalia;
	}	
}